substitute_odd_punctuation

Function substitute_odd_punctuation 

pub fn substitute_odd_punctuation(input: String) -> Result<String, Error>
product lines can contain difficult to parse punctuation such as biochemical symbols like unclosed single quotes, superscripts, single and double brackets etc. here we substitute these for an underscore